@@ -54,9 +54,12 @@ static khash_t (igkh) * ht_dates       = NULL;
 static khash_t (si32) * ht_seqs        = NULL;
 static khash_t (ss32) * ht_hostnames   = NULL;
 static khash_t (si32) * ht_cnt_overall = NULL;
-static khash_t (ii32) * ht_last_parse  = NULL;
+static khash_t (iglp) * ht_last_parse  = NULL;
 
 static GKHashModule *cache_storage = NULL;
+
+static uint32_t *persisted_dates = NULL;
+static uint32_t persisted_dates_len = 0;
 /* *INDENT-ON* */
 
 /* Allocate memory for a new store container GKHashStorage instance.
@@ -113,6 +116,13 @@ new_ii32_ht (void) {
   return h;
 }
 
+/* Initialize a new uint32_t key - GLastParse value hash table */
+static void *
+new_iglp_ht (void) {
+  khash_t (iglp) * h = kh_init (iglp);
+  return h;
+}
+
 /* Initialize a new uint32_t key - GKHashStorage value hash table */
 static void *
 new_igkh_ht (void) {
@@ -302,6 +312,15 @@ des_u648 (void *h, GO_UNUSED uint8_t free_data) {
   kh_destroy (u648, hash);
 }
 
+/* Destroys the hash structure */
+static void
+des_iglp (void *h, GO_UNUSED uint8_t free_data) {
+  khash_t (iglp) * hash = h;
+  if (!hash)
+    return;
+  kh_destroy (iglp, hash);
+}
+
 /* Deletes all entries from the hash table */
 static void
 del_u648 (void *h, GO_UNUSED uint8_t free_data) {
@@ -592,6 +611,9 @@ get_hash_from_cache (GModule module, GSMetric metric) {
   return cache_storage[module].metrics[metric].hash;
 }
 
+/* Initialize a global hash structure.
+ *
+ * On success, a pointer to that hash structure is returned. */
 static GKHashGlobal *
 init_gkhashglobal (void) {
   GKHashGlobal *storage = NULL;
@@ -608,6 +630,9 @@ init_gkhashglobal (void) {
   return storage;
 }
 
+/* Initialize a module hash structure.
+ *
+ * On success, a pointer to that hash structure is returned. */
 static GKHashModule *
 init_gkhashmodule (void) {
   GKHashModule *storage = NULL;
@@ -625,10 +650,10 @@ init_gkhashmodule (void) {
   return storage;
 }
 
-/* Insert an uint32_t key and an uint8_t value
- * Note: If the key exists, its value is replaced by the given value.
+/* Insert an uint32_t key (date) and a GKHashStorage payload
  *
  * On error, -1 is returned.
+ * On key found, 1 is returned.
  * On success 0 is returned */
 static int
 ins_igkh (khash_t (igkh) * hash, uint32_t key) {
@@ -656,6 +681,28 @@ ins_igkh (khash_t (igkh) * hash, uint32_t key) {
   return 0;
 }
 
+/* Insert an uint32_t key and an GLastParse value
+ * Note: If the key exists, its value is replaced by the given value.
+ *
+ * On error, -1 is returned.
+ * On success 0 is returned */
+static int
+ins_iglp (khash_t (iglp) * hash, uint32_t key, GLastParse lp) {
+  khint_t k;
+  int ret;
+
+  if (!hash)
+    return -1;
+
+  k = kh_put (iglp, hash, key, &ret);
+  if (ret == -1)
+    return -1;
+
+  kh_val (hash, k) = lp;
+
+  return 0;
+}
+
 /* Insert a string key and the corresponding uint32_t value.
  * Note: If the key exists, the value is not replaced.
  *
@@ -683,11 +730,11 @@ ins_si32 (khash_t (si32) * hash, const char *key, uint32_t value) {
   return 0;
 }
 
-/* Insert a string key and the corresponding uint32_t value.
+/* Increment a string key and with the corresponding incremental uint32_t value.
  * Note: If the key exists, the value is not replaced.
  *
- * On error, or if key exists, -1 is returned.
- * On success 0 is returned */
+ * On error, 0 is returned.
+ * On success or if the key exists, the value is returned */
 static uint32_t
 ins_si32_inc (khash_t (si32) * hash, const char *key, uint32_t (*cb) (const char *),
               const char *seqk) {
@@ -804,9 +851,9 @@ ins_iu64 (khash_t (iu64) * hash, uint32_t key, uint64_t value) {
 }
 
 /* Insert a string key and a uint64_t value
- * Note: If the key exists, its value is replaced by the given value.
+ * Note: If the key exists, the value is not replaced.
  *
- * On error, -1 is returned.
+ * On error or key exists, -1 is returned.
  * On success 0 is returned */
 static int
 ins_su64 (khash_t (su64) * hash, const char *key, uint64_t value) {
@@ -830,6 +877,11 @@ ins_su64 (khash_t (su64) * hash, const char *key, uint64_t value) {
   return 0;
 }
 
+/* Insert a uint64_t key and a uint8_t value
+ *
+ * On error or key exists, -1 is returned.
+ * On key exists, 1 is returned.
+ * On success 0 is returned */
 static int
 ins_u648 (khash_t (u648) * hash, uint64_t key, uint8_t value) {
   khint_t k;
@@ -938,8 +990,8 @@ inc_iu64 (khash_t (iu64) * hash, uint32_t key, uint64_t inc) {
 
 /* Increase a uint32_t value given a string key.
  *
- * On error, -1 is returned.
- * On success 0 is returned */
+ * On error, 0 is returned.
+ * On success the increased value is returned */
 static uint32_t
 inc_si32 (khash_t (si32) * hash, const char *key, uint32_t inc) {
   khint_t k;
@@ -967,7 +1019,8 @@ inc_si32 (khash_t (si32) * hash, const char *key, uint32_t inc) {
 
 /* Insert a string key and auto increment int value.
  *
- * On error, -1 is returned.
+ * On error, 0 is returned.
+ * On key found, the stored value is returned
  * On success the value of the key inserted is returned */
 static uint32_t
 ins_si32_ai (khash_t (si32) * hash, const char *key) {
@@ -1007,11 +1060,11 @@ find_int_key_in_list (void *data, void *needle) {
  * Note: If the key exists within the list, the value is not appended.
  *
  * On error, -1 is returned.
- * On success 0 is returned */
+ * On success or if key is found, 0 is returned */
 static int
 ins_igsl (khash_t (igsl) * hash, uint32_t key, uint32_t value) {
   khint_t k;
-  GSLList *list, *match;
+  GSLList *list;
   int ret;
 
   if (!hash)
@@ -1020,7 +1073,7 @@ ins_igsl (khash_t (igsl) * hash, uint32_t key, uint32_t value) {
   k = kh_get (igsl, hash, key);
   /* key found, check if key exists within the list */
   if (k != kh_end (hash) && (list = kh_val (hash, k))) {
-    if ((match = list_find (list, find_int_key_in_list, &value)))
+    if (list_find (list, find_int_key_in_list, &value))
       return 0;
     list = list_insert_prepend (list, i322ptr (value));
   } else {
@@ -1036,8 +1089,9 @@ ins_igsl (khash_t (igsl) * hash, uint32_t key, uint32_t value) {
   return 0;
 }
 
-/* Get the on-disk databases path.
+/* Determine the path for the given database file.
  *
+ * On error, a fatal error is thrown.
  * On success, the databases path string is returned. */
 static char *
 set_db_path (const char *fn) {
@@ -1067,6 +1121,10 @@ set_db_path (const char *fn) {
   return path;
 }
 
+/* Get the database filename given a module and a metric.
+ *
+ * On error, a fatal error is triggered.
+ * On success, the filename is returned */
 static char *
 get_filename (GModule module, GKHashMetric mtrc) {
   char *mtrstr = NULL, *modstr = NULL, *type = NULL, *fn = NULL;
@@ -1088,12 +1146,15 @@ get_filename (GModule module, GKHashMetric mtrc) {
   return fn;
 }
 
+/* Dump to disk the database file and frees its memory */
 static void
 close_tpl (tpl_node * tn, const char *fn) {
   tpl_dump (tn, TPL_FILE, fn);
   tpl_free (tn);
 }
 
+/* Given a database filename, restore a string key, uint32_t value back to the
+ * storage */
 static void
 restore_global_si32 (khash_t (si32) * hash, const char *fn) {
   tpl_node *tn;
@@ -1110,6 +1171,7 @@ restore_global_si32 (khash_t (si32) * hash, const char *fn) {
   tpl_free (tn);
 }
 
+/* Given a hash and a filename, persist to disk a string key, uint32_t value */
 static void
 persist_global_si32 (khash_t (si32) * hash, const char *fn) {
   tpl_node *tn;
@@ -1133,32 +1195,36 @@ persist_global_si32 (khash_t (si32) * hash, const char *fn) {
   tpl_free (tn);
 }
 
-
+/* Given a database filename, restore a uint32_t key, GLastParse value back to
+ * the storage */
 static void
-restore_global_ii32 (khash_t (ii32) * hash, const char *fn) {
+restore_global_iglp (khash_t (iglp) * hash, const char *fn) {
   tpl_node *tn;
-  uint32_t key, val;
-  char fmt[] = "A(uu)";
+  uint32_t key;
+  GLastParse val = { 0 };
+  char fmt[] = "A(uS(uIUvc#))";
 
-  tn = tpl_map (fmt, &key, &val);
+  tn = tpl_map (fmt, &key, &val, READ_BYTES);
   tpl_load (tn, TPL_FILE, fn);
   while (tpl_unpack (tn, 1) > 0) {
-    ins_ii32 (hash, key, val);
+    ins_iglp (hash, key, val);
   }
   tpl_free (tn);
 }
 
+/* Given a hash and a filename, persist to disk a uint32_t key, uint32_t value */
 static void
-persist_global_ii32 (khash_t (ii32) * hash, const char *fn) {
+persist_global_iglp (khash_t (iglp) * hash, const char *fn) {
   tpl_node *tn;
   khint_t k;
-  uint32_t key, val;
-  char fmt[] = "A(uu)";
+  uint32_t key;
+  GLastParse val = { 0 };
+  char fmt[] = "A(uS(uIUvc#))";
 
   if (!hash || kh_size (hash) == 0)
     return;
 
-  tn = tpl_map (fmt, &key, &val);
+  tn = tpl_map (fmt, &key, &val, READ_BYTES);
   for (k = 0; k < kh_end (hash); ++k) {
     if (!kh_exist (hash, k))
       continue;
@@ -1171,12 +1237,36 @@ persist_global_ii32 (khash_t (ii32) * hash, const char *fn) {
   tpl_free (tn);
 }
 
+/* Check if the given date can be inserted based on how many dates we need to
+ * keep conf.keep_last.
+ *
+ * Returns -1 if it fails to insert the date.
+ * Returns 1 if the date exists.
+ * Returns 2 if the date shouldn't be inserted.
+ * On success or if the date is inserted 0 is returned */
+static int
+insert_restored_date (uint32_t date) {
+  uint32_t i, len = 0;
+
+  /* no keep last, simply insert the restored date to our storage */
+  if (!conf.keep_last || persisted_dates_len < conf.keep_last)
+    return ht_insert_date (date);
+
+  len = MIN (persisted_dates_len, conf.keep_last);
+  for (i = 0; i < len; ++i)
+    if (persisted_dates[i] == date)
+      return ht_insert_date (date);
+  return 2;
+}
+
+/* Given a database filename, restore a string key, uint32_t value back to
+ * the storage */
 static int
 restore_si32 (GSMetric metric, const char *path, int module) {
   khash_t (si32) * hash = NULL;
   tpl_node *tn;
   char fmt[] = "A(iA(su))";
-  int date = 0;
+  int date = 0, ret = 0;
   char *key = NULL;
   uint32_t val = 0;
 
@@ -1185,9 +1275,11 @@ restore_si32 (GSMetric metric, const char *path, int module) {
 
   tpl_load (tn, TPL_FILE, path);
   while (tpl_unpack (tn, 1) > 0) {
-    /* if operation fails */
-    if ((ht_insert_date (date) == -1) || !(hash = get_hash (module, date, metric)))
-      return 1;
+    if ((ret = insert_restored_date (date)) == 2)
+      continue;
+    if (ret == -1 || !(hash = get_hash (module, date, metric)))
+      break;
+
     while (tpl_unpack (tn, 2) > 0) {
       ins_si32 (hash, key, val);
       free (key);
@@ -1198,6 +1290,7 @@ restore_si32 (GSMetric metric, const char *path, int module) {
   return 0;
 }
 
+/* Given a hash and a filename, persist to disk a string key, uint32_t value */
 static int
 persist_si32 (GSMetric metric, const char *path, int module) {
   khash_t (si32) * hash = NULL;
@@ -1223,12 +1316,14 @@ persist_si32 (GSMetric metric, const char *path, int module) {
   return 0;
 }
 
+/* Given a database filename, restore a uint32_t key, string value back to
+ * the storage */
 static int
 restore_is32 (GSMetric metric, const char *path, int module) {
   khash_t (is32) * hash = NULL;
   tpl_node *tn;
   char fmt[] = "A(iA(us))";
-  int date = 0;
+  int date = 0, ret = 0;
   uint32_t key = 0;
   char *val = NULL, *dupval = NULL;
 
@@ -1237,9 +1332,11 @@ restore_is32 (GSMetric metric, const char *path, int module) {
 
   tpl_load (tn, TPL_FILE, path);
   while (tpl_unpack (tn, 1) > 0) {
-    /* if operation fails */
-    if ((ht_insert_date (date) == -1) || !(hash = get_hash (module, date, metric)))
-      return 1;
+    if ((ret = insert_restored_date (date)) == 2)
+      continue;
+    if (ret == -1 || !(hash = get_hash (module, date, metric)))
+      break;
+
     while (tpl_unpack (tn, 2) > 0) {
       dupval = xstrdup (val);
       if (ins_is32 (hash, key, dupval) != 0)
@@ -1252,6 +1349,7 @@ restore_is32 (GSMetric metric, const char *path, int module) {
   return 0;
 }
 
+/* Given a hash and a filename, persist to disk a uint32_t key, string value */
 static int
 persist_is32 (GSMetric metric, const char *path, int module) {
   khash_t (is32) * hash = NULL;
@@ -1277,12 +1375,14 @@ persist_is32 (GSMetric metric, const char *path, int module) {
   return 0;
 }
 
+/* Given a database filename, restore a uint32_t key, uint32_t value back to
+ * the storage */
 static int
 restore_ii32 (GSMetric metric, const char *path, int module) {
   khash_t (ii32) * hash = NULL;
   tpl_node *tn;
   char fmt[] = "A(iA(uu))";
-  int date = 0;
+  int date = 0, ret = 0;
   uint32_t key = 0, val = 0;
 
   if (!(tn = tpl_map (fmt, &date, &key, &val)))
@@ -1290,9 +1390,11 @@ restore_ii32 (GSMetric metric, const char *path, int module) {
 
   tpl_load (tn, TPL_FILE, path);
   while (tpl_unpack (tn, 1) > 0) {
-    /* if operation fails */
-    if ((ht_insert_date (date) == -1) || !(hash = get_hash (module, date, metric)))
-      return 1;
+    if ((ret = insert_restored_date (date)) == 2)
+      continue;
+    if (ret == -1 || !(hash = get_hash (module, date, metric)))
+      break;
+
     while (tpl_unpack (tn, 2) > 0) {
       ins_ii32 (hash, key, val);
     }
@@ -1302,6 +1404,7 @@ restore_ii32 (GSMetric metric, const char *path, int module) {
   return 0;
 }
 
+/* Given a hash and a filename, persist to disk a uint32_t key, uint32_t value */
 static int
 persist_ii32 (GSMetric metric, const char *path, int module) {
   khash_t (ii32) * hash = NULL;
@@ -1326,12 +1429,14 @@ persist_ii32 (GSMetric metric, const char *path, int module) {
   return 0;
 }
 
+/* Given a database filename, restore a uint64_t key, uint8_t value back to
+ * the storage */
 static int
 restore_u648 (GSMetric metric, const char *path, int module) {
   khash_t (u648) * hash = NULL;
   tpl_node *tn;
   char fmt[] = "A(iA(Uv))";
-  int date = 0;
+  int date = 0, ret = 0;
   uint64_t key;
   uint16_t val = 0;
 
@@ -1340,9 +1445,11 @@ restore_u648 (GSMetric metric, const char *path, int module) {
 
   tpl_load (tn, TPL_FILE, path);
   while (tpl_unpack (tn, 1) > 0) {
-    /* if operation fails */
-    if ((ht_insert_date (date) == -1) || !(hash = get_hash (module, date, metric)))
-      return 1;
+    if ((ret = insert_restored_date (date)) == 2)
+      continue;
+    if (ret == -1 || !(hash = get_hash (module, date, metric)))
+      break;
+
     while (tpl_unpack (tn, 2) > 0) {
       ins_u648 (hash, key, val);
     }
@@ -1352,6 +1459,7 @@ restore_u648 (GSMetric metric, const char *path, int module) {
   return 0;
 }
 
+/* Given a hash and a filename, persist to disk a uint64_t key, uint8_t value */
 static int
 persist_u648 (GSMetric metric, const char *path, int module) {
   khash_t (u648) * hash = NULL;
@@ -1377,12 +1485,14 @@ persist_u648 (GSMetric metric, const char *path, int module) {
   return 0;
 }
 
+/* Given a database filename, restore a uint32_t key, uint64_t value back to
+ * the storage */
 static int
 restore_iu64 (GSMetric metric, const char *path, int module) {
   khash_t (iu64) * hash = NULL;
   tpl_node *tn;
   char fmt[] = "A(iA(uU))";
-  int date = 0;
+  int date = 0, ret = 0;
   uint32_t key;
   uint64_t val;
 
@@ -1391,9 +1501,11 @@ restore_iu64 (GSMetric metric, const char *path, int module) {
 
   tpl_load (tn, TPL_FILE, path);
   while (tpl_unpack (tn, 1) > 0) {
-    /* if operation fails */
-    if ((ht_insert_date (date) == -1) || !(hash = get_hash (module, date, metric)))
-      return 1;
+    if ((ret = insert_restored_date (date)) == 2)
+      continue;
+    if (ret == -1 || !(hash = get_hash (module, date, metric)))
+      break;
+
     while (tpl_unpack (tn, 2) > 0) {
       ins_iu64 (hash, key, val);
     }
@@ -1403,6 +1515,7 @@ restore_iu64 (GSMetric metric, const char *path, int module) {
   return 0;
 }
 
+/* Given a hash and a filename, persist to disk a uint32_t key, uint64_t value */
 static int
 persist_iu64 (GSMetric metric, const char *path, int module) {
   khash_t (iu64) * hash = NULL;
@@ -1428,12 +1541,14 @@ persist_iu64 (GSMetric metric, const char *path, int module) {
   return 0;
 }
 
+/* Given a database filename, restore a string key, uint64_t value back to
+ * the storage */
 static int
 restore_su64 (GSMetric metric, const char *path, int module) {
   khash_t (su64) * hash = NULL;
   tpl_node *tn;
   char fmt[] = "A(iA(sU))";
-  int date = 0;
+  int date = 0, ret = 0;
   char *key = NULL;
   uint64_t val;
 
@@ -1442,9 +1557,11 @@ restore_su64 (GSMetric metric, const char *path, int module) {
 
   tpl_load (tn, TPL_FILE, path);
   while (tpl_unpack (tn, 1) > 0) {
-    /* if operation fails */
-    if ((ht_insert_date (date) == -1) || !(hash = get_hash (module, date, metric)))
-      return 1;
+    if ((ret = insert_restored_date (date)) == 2)
+      continue;
+    if (ret == -1 || !(hash = get_hash (module, date, metric)))
+      break;
+
     while (tpl_unpack (tn, 2) > 0) {
       ins_su64 (hash, key, val);
       free (key);
@@ -1455,6 +1572,7 @@ restore_su64 (GSMetric metric, const char *path, int module) {
   return 0;
 }
 
+/* Given a hash and a filename, persist to disk a string key, uint64_t value */
 static int
 persist_su64 (GSMetric metric, const char *path, int module) {
   khash_t (su64) * hash = NULL;
@@ -1480,12 +1598,14 @@ persist_su64 (GSMetric metric, const char *path, int module) {
   return 0;
 }
 
+/* Given a database filename, restore a uint32_t key, GSLList value back to the
+ * storage */
 static int
 restore_igsl (GSMetric metric, const char *path, int module) {
   khash_t (igsl) * hash = NULL;
   tpl_node *tn;
   char fmt[] = "A(iA(uu))";
-  int date = 0;
+  int date = 0, ret = 0;
   uint32_t key, val;
 
   if (!(tn = tpl_map (fmt, &date, &key, &val)))
@@ -1493,9 +1613,11 @@ restore_igsl (GSMetric metric, const char *path, int module) {
 
   tpl_load (tn, TPL_FILE, path);
   while (tpl_unpack (tn, 1) > 0) {
-    /* if operation fails */
-    if ((ht_insert_date (date) == -1) || !(hash = get_hash (module, date, metric)))
-      return 1;
+    if ((ret = insert_restored_date (date)) == 2)
+      continue;
+    if (ret == -1 || !(hash = get_hash (module, date, metric)))
+      break;
+
     while (tpl_unpack (tn, 2) > 0) {
       ins_igsl (hash, key, val);
     }
@@ -1505,6 +1627,7 @@ restore_igsl (GSMetric metric, const char *path, int module) {
   return 0;
 }
 
+/* Given a hash and a filename, persist to disk a uint32_t key, GSLList value */
 static int
 persist_igsl (GSMetric metric, const char *path, int module) {
   khash_t (igsl) * hash = NULL;
@@ -1536,6 +1659,10 @@ persist_igsl (GSMetric metric, const char *path, int module) {
   return 0;
 }
 
+/* Given a filename, ensure we have a valid return path
+ *
+ * On error, NULL is returned.
+ * On success, the valid path is returned */
 static char *
 check_restore_path (const char *fn) {
   char *path = set_db_path (fn);
@@ -1547,6 +1674,7 @@ check_restore_path (const char *fn) {
   return NULL;
 }
 
+/* Entry function to restore hash data by type */
 static void
 restore_by_type (GKHashMetric mtrc, const char *fn, int module) {
   char *path = NULL;
@@ -1583,6 +1711,7 @@ clean:
   free (path);
 }
 
+/* Entry function to restore hash data by metric type */
 static void
 restore_metric_type (GModule module, GKHashMetric mtrc) {
   char *fn = NULL;
@@ -1592,20 +1721,76 @@ restore_metric_type (GModule module, GKHashMetric mtrc) {
   free (fn);
 }
 
+/* Given all the dates that we have processed, persist to disk a copy of them. */
+static void
+persist_dates (void) {
+  tpl_node *tn;
+  char *path = NULL;
+  uint32_t *dates = NULL, len = 0, i, date = 0;
+  char fmt[] = "A(u)";
+
+  if (!(path = set_db_path ("I32_DATES.db")))
+    return;
+
+  dates = get_sorted_dates (&len);
+
+  tn = tpl_map (fmt, &date);
+  for (i = 0; i < len; ++i) {
+    date = dates[i];
+    tpl_pack (tn, 1);
+  }
+  tpl_dump (tn, TPL_FILE, path);
+
+  tpl_free (tn);
+  free (path);
+  free (dates);
+}
+
+/* Restore all the processed dates from our last dataset */
+static void
+restore_dates (void) {
+  tpl_node *tn;
+  char *path = NULL;
+  uint32_t date, idx = 0;
+  char fmt[] = "A(u)";
+
+  if (!(path = check_restore_path ("I32_DATES.db")))
+    return;
+
+  tn = tpl_map (fmt, &date);
+  tpl_load (tn, TPL_FILE, path);
+
+  persisted_dates_len = tpl_Alen (tn, 1);
+  persisted_dates = xcalloc (persisted_dates_len, sizeof (uint32_t));
+  while (tpl_unpack (tn, 1) > 0)
+    persisted_dates[idx++] = date;
+
+  qsort (persisted_dates, idx, sizeof (uint32_t), cmp_ui32_desc);
+  tpl_free (tn);
+  free (path);
+}
+
+/* Entry function to restore a global hashes */
 static void
 restore_global (void) {
   char *path = NULL;
 
+  restore_dates ();
   if ((path = check_restore_path ("SI32_CNT_OVERALL.db"))) {
     restore_global_si32 (ht_cnt_overall, path);
     free (path);
   }
-  if ((path = check_restore_path ("II32_LAST_PARSE.db"))) {
-    restore_global_ii32 (ht_last_parse, path);
+  if ((path = check_restore_path ("SI32_SEQS.db"))) {
+    restore_global_si32 (ht_seqs, path);
+    free (path);
+  }
+  if ((path = check_restore_path ("IGLP_LAST_PARSE.db"))) {
+    restore_global_iglp (ht_last_parse, path);
     free (path);
   }
 }
 
+/* Entry function to restore hashes */
 static void
 restore_data (void) {
   GModule module;
@@ -1672,14 +1857,20 @@ static void
 persist_global (void) {
   char *path = NULL;
 
+  persist_dates ();
   if ((path = set_db_path ("SI32_CNT_OVERALL.db"))) {
     persist_global_si32 (ht_cnt_overall, path);
     free (path);
   }
-  if ((path = set_db_path ("II32_LAST_PARSE.db"))) {
-    persist_global_ii32 (ht_last_parse, path);
+  if ((path = set_db_path ("SI32_SEQS.db"))) {
+    persist_global_si32 (ht_seqs, path);
     free (path);
   }
+  if ((path = set_db_path ("IGLP_LAST_PARSE.db"))) {
+    persist_global_iglp (ht_last_parse, path);
+    free (path);
+  }
+  free (persisted_dates);
 }
 
 static void
@@ -1823,6 +2014,29 @@ get_su64 (khash_t (su64) * hash, const char *key) {
   return 0;
 }
 
+/* Get the GLastParse value of a given uint32_t key.
+ *
+ * If key is not found, {0} is returned.
+ * On error, -1 is returned.
+ * On success the GLastParse value for the given key is returned */
+static GLastParse
+get_iglp (khash_t (iglp) * hash, uint32_t key) {
+  khint_t k;
+  GLastParse lp = { 0 };
+
+  if (!hash)
+    return lp;
+
+  k = kh_get (iglp, hash, key);
+  /* key found, return current value */
+  if (k != kh_end (hash)) {
+    lp = kh_val (hash, k);
+    return lp;
+  }
+
+  return lp;
+}
+
 GSLList *
 ht_get_keymap_list_from_key (GModule module, const char *key) {
   GSLList *list = NULL;
@@ -1996,13 +2210,13 @@ ht_sum_bw (void) {
 }
 
 int
-ht_insert_last_parse (uint32_t key, uint32_t value) {
-  khash_t (ii32) * hash = ht_last_parse;
+ht_insert_last_parse (uint32_t key, GLastParse lp) {
+  khash_t (iglp) * hash = ht_last_parse;
 
   if (!hash)
     return 0;
 
-  return ins_ii32 (hash, key, value);
+  return ins_iglp (hash, key, lp);
 }
 
 int
@@ -2010,7 +2224,7 @@ ht_insert_date (uint32_t key) {
   khash_t (igkh) * hash = ht_dates;
 
   if (!hash)
-    return 0;
+    return -1;
 
   return ins_igkh (hash, key);
 }
@@ -2074,6 +2288,7 @@ u64encode (uint32_t x, uint32_t y) {
   return x > y ? (uint32_t) y | ((uint64_t) x << 32) : (uint32_t) x | ((uint64_t) y << 32);
 }
 
+/* Decode a uint64_t number into the original two uint32_t  */
 void
 u64decode (uint64_t n, uint32_t * x, uint32_t * y) {
   *x = (uint64_t) n >> 32;
@@ -2337,15 +2552,14 @@ int
 ht_insert_maxts (GModule module, uint32_t date, uint32_t key, uint64_t value, uint32_t ckey) {
   khash_t (iu64) * hash = get_hash (module, date, MTRC_MAXTS);
   khash_t (iu64) * cache = get_hash_from_cache (module, MTRC_MAXTS);
-  uint64_t curvalue = 0;
 
   if (!hash)
     return -1;
 
-  if ((curvalue = get_iu64 (hash, key)) < value) {
+  if (get_iu64 (cache, ckey) < value)
     ins_iu64 (cache, ckey, value);
+  if (get_iu64 (hash, key) < value)
     ins_iu64 (hash, key, value);
-  }
 
   return 0;
 }
@@ -2440,14 +2654,10 @@ ht_insert_hostname (const char *ip, const char *host) {
   return ins_ss32 (hash, ip, host);
 }
 
-uint32_t
+GLastParse
 ht_get_last_parse (uint32_t key) {
-  khash_t (ii32) * hash = ht_last_parse;
-
-  if (!hash)
-    return 0;
-
-  return get_ii32 (hash, key);
+  khash_t (iglp) * hash = ht_last_parse;
+  return get_iglp (hash, key);
 }
 
 /* Get the number of elements in a datamap.
@@ -2820,10 +3030,19 @@ get_sorted_dates (uint32_t * len) {
   return dates;
 }
 
+void
+destroy_date_stores (int date) {
+  khash_t (igkh) * hash = ht_dates;
+  khiter_t k;
+
+  k = kh_get (igkh, hash, date);
+  free_stores (kh_value (hash, k));
+  kh_del (igkh, hash, k);
+}
+
 int
 invalidate_date (int date) {
   khash_t (igkh) * hash = ht_dates;
-  khiter_t k;
   GModule module;
   size_t idx = 0;
 
@@ -2835,9 +3054,7 @@ invalidate_date (int date) {
     del_module_metrics (cache_storage, module, 0);
   }
 
-  k = kh_get (igkh, hash, date);
-  free_stores (kh_value (hash, k));
-  kh_del (igkh, hash, k);
+  destroy_date_stores (date);
 
   return 0;
 }
@@ -2875,6 +3092,21 @@ inc_cache_ii32 (GKHashStorage * store, GModule module, GSMetric metric, uint32_t
   return inc_ii32 (cache, ckey, kh_val (hash, k));
 }
 
+static int
+max_cache_iu64 (GKHashStorage * store, GModule module, GSMetric metric, uint32_t key,
+                uint32_t ckey) {
+  khash_t (iu64) * hash = get_hash_from_store (store, module, metric);
+  khash_t (iu64) * cache = get_hash_from_cache (module, metric);
+  khint_t k;
+
+  if ((k = kh_get (iu64, hash, key)) == kh_end (hash))
+    return -1;
+
+  if (get_iu64 (cache, ckey) < kh_val (hash, k))
+    return ins_iu64 (cache, ckey, kh_val (hash, k));
+  return -1;
+}
+
 static int
 inc_cache_iu64 (GKHashStorage * store, GModule module, GSMetric metric, uint32_t key,
                 uint32_t ckey) {
@@ -2922,7 +3154,7 @@ ins_raw_num_data (GModule module, uint32_t date) {
     inc_cache_ii32 (store, module, MTRC_VISITORS, kh_val (kmap, k), ckey);
     inc_cache_iu64 (store, module, MTRC_BW, kh_val (kmap, k), ckey);
     inc_cache_iu64 (store, module, MTRC_CUMTS, kh_val (kmap, k), ckey);
-    inc_cache_iu64 (store, module, MTRC_MAXTS, kh_val (kmap, k), ckey);
+    max_cache_iu64 (store, module, MTRC_MAXTS, kh_val (kmap, k), ckey);
     ins_cache_is32 (store, module, MTRC_METHODS, kh_val (kmap, k), ckey);
     ins_cache_is32 (store, module, MTRC_PROTOCOLS, kh_val (kmap, k), ckey);
   }
@@ -3046,11 +3278,13 @@ parse_raw_data (GModule module) {
   switch (module) {
   case VISITORS:
     raw_data = get_str_raw_data (module);
-    sort_raw_str_data (raw_data, raw_data->idx);
+    if (raw_data)
+      sort_raw_str_data (raw_data, raw_data->idx);
     break;
   default:
     raw_data = get_u32_raw_data (module);
-    sort_raw_num_data (raw_data, raw_data->idx);
+    if (raw_data)
+      sort_raw_num_data (raw_data, raw_data->idx);
   }
 
   modstr = get_module_str (module);
@@ -3069,7 +3303,7 @@ init_storage (void) {
   ht_dates       = (khash_t (igkh) *) new_igkh_ht ();
   ht_seqs        = (khash_t (si32) *) new_si32_ht ();
   ht_cnt_overall = (khash_t (si32) *) new_si32_ht ();
-  ht_last_parse  = (khash_t (ii32) *) new_ii32_ht ();
+  ht_last_parse  = (khash_t (iglp) *) new_iglp_ht ();
   /* *INDENT-ON* */
 
   cache_storage = init_gkhashmodule ();
@@ -3115,5 +3349,5 @@ free_storage (void) {
   free (cache_storage);
 
   des_si32_free (ht_cnt_overall, 1);
-  des_ii32 (ht_last_parse, 1);
+  des_iglp (ht_last_parse, 1);
 }
