@@ -54,7 +54,8 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
-#include <malloc.h>
+#include <sys/stat.h>
+#include <fcntl.h>
 
 #include "gkhash.h"
 
@@ -438,7 +439,7 @@ init_log_item (GLog * glog) {
   logitem->cache_status = NULL;
 
   memset (logitem->site, 0, sizeof (logitem->site));
-  logitem->dt = *localtime (&now);
+  localtime_r (&now, &logitem->dt);
 
   return logitem;
 }
@@ -534,7 +535,7 @@ decode_url (char *url) {
     decode_hex (decoded, out);
   strip_newlines (out);
 
-  return trim_str (out);
+  return trim_str (char_replace (out, '+', ' '));
 }
 
 /* Process keyphrases from Google search, cache, and translate.
@@ -735,17 +736,19 @@ contains_usecs (void) {
 
 static int
 is_cache_hit (const char *tkn) {
-  const char *statuses[] = {
-    "MISS",
-    "BYPASS",
-    "EXPIRED",
-    "STALE",
-    "UPDATING",
-    "REVALIDATED",
-    "HIT",
-  };
-
-  if (str_inarray (tkn, statuses, CACHE_STATUS_LEN) != -1)
+  if (strcasecmp ("MISS", tkn) == 0)
+    return 1;
+  else if (strcasecmp ("BYPASS", tkn) == 0)
+    return 1;
+  else if (strcasecmp ("EXPIRED", tkn) == 0)
+    return 1;
+  else if (strcasecmp ("STALE", tkn) == 0)
+    return 1;
+  else if (strcasecmp ("UPDATING", tkn) == 0)
+    return 1;
+  else if (strcasecmp ("REVALIDATED", tkn) == 0)
+    return 1;
+  else if (strcasecmp ("HIT", tkn) == 0)
     return 1;
   return 0;
 }
@@ -1051,8 +1054,8 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     if (!(tkn = parse_string (&(*str), end, 1)))
       return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
 
-    if (str_to_time (tkn, tfmt, &tm) != 0 || set_date (&logitem->date, tm) != 0
-        || set_time (&logitem->time, tm) != 0) {
+    if (str_to_time (tkn, tfmt, &tm) != 0 || set_date (&logitem->date, tm) != 0 ||
+        set_time (&logitem->time, tm) != 0) {
       spec_err (logitem, SPEC_TOKN_INV, *p, tkn);
       free (tkn);
       return 1;
@@ -1088,6 +1091,8 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
       return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
     if (is_cache_hit (tkn))
       logitem->cache_status = tkn;
+    else
+      free (tkn);
     break;
     /* remote hostname (IP only) */
   case 'h':
@@ -1633,6 +1638,12 @@ count_process (GLog * glog) {
   unlock_spinner ();
 }
 
+static void
+count_process_and_invalid (GLog * glog, const char *line) {
+  count_process (glog);
+  count_invalid (glog, line);
+}
+
 /* Keep track of all excluded log strings (IPs).
  *
  * If IP not range, 1 is returned.
@@ -1723,7 +1734,7 @@ ignore_line (GLogItem * logitem) {
     return IGNORE_LEVEL_PANEL;
   if (handle_crawler (logitem->agent) == 0)
     return IGNORE_LEVEL_PANEL;
-  if (ignore_referer (logitem->site))
+  if (ignore_referer (logitem->ref))
     return IGNORE_LEVEL_PANEL;
   if (ignore_status_code (logitem->status))
     return IGNORE_LEVEL_PANEL;
@@ -2338,7 +2349,7 @@ gen_visit_time_key (GKeyData * kdata, GLogItem * logitem) {
   /* otherwise it attempts to convert the date given a time format,
    * though this is slower */
   memset (hour, 0, sizeof *hour);
-  if (convert_date (hour, logitem->time, conf.time_format, "%H:%M", HRMI_LEN) != 0)
+  if (convert_date (hour, logitem->time, "%T", "%H:%M", HRMI_LEN) != 0)
     return 1;
 
   if (*hour == '\0')
@@ -2445,20 +2456,39 @@ ins_agent_key_val (GLogItem * logitem, uint32_t numdate) {
   }
 }
 
-static void
-clean_old_data_by_date (void) {
+static int
+clean_old_data_by_date (uint32_t numdate) {
   uint32_t *dates = NULL;
-  uint32_t len = 0;
+  uint32_t idx, len = 0;
 
-  if (ht_get_size_dates () <= conf.keep_last)
-    return;
+  if (ht_get_size_dates () < conf.keep_last)
+    return 1;
 
   dates = get_sorted_dates (&len);
+
+  /* If currently parsed date is in the set of dates, keep inserting it.
+   * We count down since more likely the currently parsed date is at the last pos */
+  for (idx = len; idx-- > 0;) {
+    if (dates[idx] == numdate) {
+      free (dates);
+      return 1;
+    }
+  }
+
+  /* ignore older dates */
+  if (dates[0] > numdate) {
+    free (dates);
+    return -1;
+  }
+
+  /* invalidate the first date we inserted then */
   invalidate_date (dates[0]);
   /* rebuild all existing dates and let new data
    * be added upon existing cache */
   rebuild_rawdata_cache ();
   free (dates);
+
+  return 0;
 }
 
 /* Process a log line and set the data into the corresponding data
@@ -2470,14 +2500,13 @@ process_log (GLogItem * logitem) {
   size_t idx = 0;
   uint32_t numdate = logitem->numdate;
 
+  if (conf.keep_last > 0 && clean_old_data_by_date (numdate) == -1)
+    return;
 
   /* insert date and start partitioning tables */
   if (ht_insert_date (numdate) == -1)
     return;
 
-  if (conf.keep_last > 0)
-    clean_old_data_by_date ();
-
   /* Insert one unique visitor key per request to avoid the
    * overhead of storing one key per module */
   if ((logitem->uniq_nkey = ht_insert_unique_key (numdate, logitem->uniq_key)) == 0)
@@ -2503,6 +2532,118 @@ process_log (GLogItem * logitem) {
     count_valid (numdate);
 }
 
+/* Determine if the current log has the content from the last time it was
+ * parsed. It does this by comparing READ_BYTES against the beginning of the
+ * log.
+ *
+ * Returns 1 if the content is likely the same or no data to compare
+ * Returns 0 if it has different content */
+static int
+is_likely_same_log (GLog * glog, GLastParse lp) {
+  size_t size = 0;
+
+  if (!lp.size)
+    return 1;
+
+  /* Must be a LOG */
+  size = MIN (glog->snippetlen, lp.snippetlen);
+  if (glog->snippet[0] != '\0' && lp.snippet[0] != '\0' &&
+      memcmp (glog->snippet, lp.snippet, size) == 0)
+    return 1;
+
+  return 0;
+}
+
+/* Determine if we should insert new record or if it's a duplicate record from
+ * a previoulsy persisted dataset
+ *
+ * Returns 1 if it thinks the record it's being restored from disk
+ * Returns 0 if we need to parse the record */
+static int
+should_restore_from_disk (GLog * glog) {
+  GLastParse lp = { 0 };
+
+  if (!conf.restore)
+    return 0;
+
+  lp = ht_get_last_parse (glog->inode);
+
+  /* No last parse timestamp, continue parsing as we got nothing to compare
+   * against */
+  if (!lp.ts)
+    return 0;
+
+  /* If our current line is greater or equal (zero indexed) to the last parsed
+   * line and have equal timestamps, then keep parsing then */
+  if (glog->inode && is_likely_same_log (glog, lp)) {
+    if (glog->size > lp.size && glog->read >= lp.line)
+      return 0;
+    return 1;
+  }
+
+  /* No inode (probably a pipe), prior or equal timestamps means restore from
+   * disk (exclusive) */
+  if (!glog->inode && lp.ts >= glog->lp.ts)
+    return 1;
+
+  /* If not likely the same content, then fallback to the following checks */
+  /* If timestamp is greater than last parsed, read the line then */
+  if (glog->lp.ts > lp.ts)
+    return 0;
+
+  /* Check if current log size is smaller than the one last parsed, if it is,
+   * it was possibly truncated and thus it may be smaller, so fallback to
+   * timestamp even if they are equal to the last parsed timestamp */
+  else if (glog->size < lp.size && glog->lp.ts == lp.ts)
+    return 0;
+
+  /* Everything else we ignore it. For instance, we if current log size is
+   * greater than the one last parsed, if the timestamp are equal, we ignore the
+   * request.
+   *
+   * **NOTE* We try to play safe here as we would rather miss a few lines
+   * than double-count a few. */
+  return 1;
+}
+
+static void
+process_invalid (GLog * glog, GLogItem * logitem, const char *line) {
+  GLastParse lp = { 0 };
+
+  /* if not restoring from disk, then count entry as proceeded and invalid */
+  if (!conf.restore) {
+    count_process_and_invalid (glog, line);
+    return;
+  }
+
+  lp = ht_get_last_parse (glog->inode);
+
+  /* If our current line is greater or equal (zero indexed) to the last parsed
+   * line then keep parsing then */
+  if (glog->inode && is_likely_same_log (glog, lp)) {
+    /* only count invalids if we're past the last parsed line */
+    if (glog->size > lp.size && glog->read >= lp.line)
+      count_process_and_invalid (glog, line);
+    return;
+  }
+
+  /* no timestamp to compare against, just count the invalid then */
+  if (!logitem->numdate) {
+    count_process_and_invalid (glog, line);
+    return;
+  }
+
+  /* if there's a valid timestamp, count only if greater than last parsed ts */
+  if ((glog->lp.ts = mktime (&logitem->dt)) == -1)
+    return;
+
+  /* check if we were able to at least parse the date/time, if no date/time
+   * then we simply don't count the entry as proceed & invalid to attempt over
+   * counting restored data */
+  if (should_restore_from_disk (glog) == 0)
+    count_process_and_invalid (glog, line);
+}
+
 /* Process a line from the log and store it accordingly taking into
  * account multiple parsing options prior to setting data into the
  * corresponding data structure.
@@ -2512,30 +2653,25 @@ static int
 pre_process_log (GLog * glog, char *line, int dry_run) {
   GLogItem *logitem;
   int ret = 0;
-  uint32_t last = ht_get_last_parse (glog->inode);
-  uint32_t ts = 0;
-
-  /* if it's a log, then use the last parsed line */
-  if (glog->inode > 0 && last > 0 && last > glog->read)
-    return 0;
 
   /* soft ignore these lines */
   if (valid_line (line))
     return -1;
 
-  count_process (glog);
   logitem = init_log_item (glog);
   /* Parse a line of log, and fill structure with appropriate values */
-  if (parse_format (logitem, line) || verify_missing_fields (logitem)) {
-    ret = 1;
-    count_invalid (glog, line);
+  if ((ret = parse_format (logitem, line)) || (ret = verify_missing_fields (logitem))) {
+    process_invalid (glog, logitem, line);
     goto cleanup;
   }
 
-  /* it's a pipe, then use the last parsed timestamp */
-  ts = mktime (&logitem->dt);
-  if (conf.restore && !glog->inode && last > 0 && last >= ts)
-    return 0;
+  if ((glog->lp.ts = mktime (&logitem->dt)) == -1)
+    goto cleanup;
+
+  if (should_restore_from_disk (glog))
+    goto cleanup;
+
+  count_process (glog);
 
   /* agent will be null in cases where %u is not specified */
   if (logitem->agent == NULL)
@@ -2562,8 +2698,6 @@ pre_process_log (GLog * glog, char *line, int dry_run) {
 cleanup:
   free_glog (logitem);
 
-  ht_insert_last_parse (0, ts);
-
   return ret;
 }
 
@@ -2574,7 +2708,6 @@ cleanup:
 static int
 read_line (GLog * glog, char *line, int *test, int *cnt, int dry_run) {
   int ret = 0;
-  int tests = conf.num_tests;
 
   /* start processing log line */
   if ((ret = pre_process_log (glog, line, dry_run)) == 0 && *test)
@@ -2584,13 +2717,9 @@ read_line (GLog * glog, char *line, int *test, int *cnt, int dry_run) {
   if (ret == -1)
     return 0;
 
-  /* glog->processed can actually be less than conf.num_tests, so we make sure
-   * (cnt) compares to the right number */
-  tests = MIN (conf.num_tests, glog->processed);
-
   /* reached num of lines to test and no valid records were found, log
    * format is likely not matching */
-  if (conf.num_tests && ++(*cnt) == tests && *test) {
+  if (conf.num_tests && ++(*cnt) == (int) conf.num_tests && *test) {
     uncount_processed (glog);
     uncount_invalid (glog);
     return 1;
@@ -2651,6 +2780,7 @@ read_lines (FILE * fp, GLog ** glog, int dry_run) {
   char *line = NULL;
   int ret = 0, cnt = 0, test = conf.num_tests > 0 ? 1 : 0;
 
+  (*glog)->bytes = 0;
   while ((line = fgetline (fp)) != NULL) {
     /* handle SIGINT */
     if (conf.stop_processing)
@@ -2659,6 +2789,7 @@ read_lines (FILE * fp, GLog ** glog, int dry_run) {
       goto out;
     if (dry_run && NUM_TESTS == cnt)
       goto out;
+    (*glog)->bytes += strlen (line);
     free (line);
     (*glog)->read++;
   }
@@ -2668,11 +2799,15 @@ read_lines (FILE * fp, GLog ** glog, int dry_run) {
   if (!line && (errno == EAGAIN || errno == EWOULDBLOCK) && test)
     return 0;
 
-  return (line && test) || ret;
+  return (line && test) || ret || (!line && test && (*glog)->processed);
 
 out:
   free (line);
-  return test || ret;
+  /* fails if
+     - we're still reading the log but the test flag was still set
+     - ret flag is not 0, read_line failed
+     - reached the end of file, test flag was still set and we processed lines */
+  return test || ret || (test && (*glog)->processed);
 }
 #endif
 
@@ -2687,6 +2822,7 @@ read_lines (FILE * fp, GLog ** glog, int dry_run) {
   char line[LINE_BUFFER] = { 0 };
   int ret = 0, cnt = 0, test = conf.num_tests > 0 ? 1 : 0;
 
+  (*glog)->bytes = 0;
   while ((s = fgets (line, LINE_BUFFER, fp)) != NULL) {
     /* handle SIGINT */
     if (conf.stop_processing)
@@ -2695,6 +2831,7 @@ read_lines (FILE * fp, GLog ** glog, int dry_run) {
       break;
     if (dry_run && NUM_TESTS == cnt)
       break;
+    (*glog)->bytes += strlen (line);
     (*glog)->read++;
   }
 
@@ -2703,10 +2840,53 @@ read_lines (FILE * fp, GLog ** glog, int dry_run) {
   if (!s && (errno == EAGAIN || errno == EWOULDBLOCK) && test)
     return 0;
 
-  return (s && test) || ret;
+  /* fails if
+     - we're still reading the log but the test flag was still set
+     - ret flag is not 0, read_line failed
+     - reached the end of file, test flag was still set and we processed lines */
+  return (s && test) || ret || (!s && test && (*glog)->processed);
 }
 #endif
 
+/* Read the given log file and attempt to mmap a fixed number of bytes so we
+ * can compare its content on future runs.
+ *
+ * On error, 1 is returned.
+ * On success, 0 is returned. */
+static int
+set_initial_persisted_data (GLog * glog, FILE * fp, const char *fn) {
+  size_t len;
+
+  if (glog->size == 0)
+    return 1;
+
+  len = MIN (glog->size, READ_BYTES);
+  if ((fread (glog->snippet, len, 1, fp)) != 1 && ferror (fp))
+    FATAL ("Unable to fread the specified log file '%s'", fn);
+  glog->snippetlen = len;
+
+  fseek (fp, 0, SEEK_SET);
+
+  return 0;
+}
+
+static void
+persist_last_parse (GLog * glog) {
+  /* insert last parsed data for the recently file parsed */
+  if (glog->inode && glog->size) {
+    glog->lp.line = glog->read;
+    glog->lp.snippetlen = glog->snippetlen;
+
+    memcpy (glog->lp.snippet, glog->snippet, glog->snippetlen);
+
+    ht_insert_last_parse (glog->inode, glog->lp);
+  }
+  /* probably from a pipe */
+  else if (!glog->inode) {
+    ht_insert_last_parse (0, glog->lp);
+  }
+}
+
 /* Read the given log line by line and process its data.
  *
  * On error, 1 is returned.
@@ -2727,11 +2907,15 @@ read_log (GLog ** glog, const char *fn, int dry_run) {
 
   /* make sure we can open the log (if not reading from stdin) */
   if (!piping && (fp = fopen (fn, "r")) == NULL)
-    FATAL ("Unable to open the specified log file. %s", strerror (errno));
+    FATAL ("Unable to open the specified log file '%s'. %s", fn, strerror (errno));
 
   /* grab the inode of the file being parsed */
-  if (!piping && stat (fn, &fdstat) == 0)
+  if (!piping && stat (fn, &fdstat) == 0) {
     (*glog)->inode = fdstat.st_ino;
+    (*glog)->size = (*glog)->lp.size = fdstat.st_size;
+
+    set_initial_persisted_data ((*glog), fp, fn);
+  }
 
   /* read line by line */
   if (read_lines (fp, glog, dry_run)) {
@@ -2740,9 +2924,7 @@ read_log (GLog ** glog, const char *fn, int dry_run) {
     return 1;
   }
 
-  /* insert the inode of the file parsed and the last line parsed */
-  if ((*glog)->inode)
-    ht_insert_last_parse ((*glog)->inode, (*glog)->read);
+  persist_last_parse ((*glog));
 
   /* close log file if not a pipe */
   if (!piping)
@@ -2787,6 +2969,7 @@ parse_log (GLog ** glog, char *tail, int dry_run) {
       fprintf (stderr, "%s\n", conf.filenames[i]);
       return 1;
     }
+    (*glog)->filesizes[i] = (*glog)->bytes;
   }
 
   return 0;
