@@ -35,6 +35,7 @@
 #define LINE_BUFFER     4096    /* read at most this num of chars */
 #define NUM_TESTS       20      /* test this many lines from the log */
 #define MAX_LOG_ERRORS  20
+#define READ_BYTES      4096u
 
 #define LINE_LEN        23
 #define ERROR_LEN       255
@@ -91,6 +92,14 @@ typedef struct GLogItem_ {
   struct tm dt;
 } GLogItem;
 
+typedef struct GLastParse_ {
+  uint32_t line;
+  int64_t ts;
+  uint64_t size;
+  uint16_t snippetlen;
+  char snippet[READ_BYTES + 1];
+} GLastParse;
+
 /* Overall parsed log properties */
 typedef struct GLog_ {
   unsigned int invalid;
@@ -101,12 +110,21 @@ typedef struct GLog_ {
   unsigned short piping;
   uint32_t read;                /* lines read/parsed */
   uint32_t inode;
+  uint64_t bytes;               /* bytes read */
+  uint64_t size;                /* bytes read */
+
+  /* file test for persisted/restored data */
+  uint16_t snippetlen;
+  char snippet[READ_BYTES + 1];
 
   GLogItem *items;
+  GLastParse lp;
 
   unsigned short log_erridx;
   char **errors;
 
+  uint64_t *filesizes;          /* log size/bytes already read */
+
   FILE *pipe;
 } GLog;
 
