@@ -148,9 +148,6 @@
 #define ERR_WIN_HEIGHT    15
 #define ERR_WIN_WIDTH     61
 
-/* Convenient macros */
-#define MIN(a, b) (((a) < (b)) ? (a) : (b))
-
 #include "color.h"
 #include "commons.h"
 #include "sort.h"
